# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Fabio Massaioli fabio.massaioli@virgilio.it
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk, GLib, Gio # pylint: disable=E0611

from lockbox_lib.helpers import get_builder
import diskctl, MessageBox

import gettext
from gettext import gettext as _
gettext.textdomain('lockbox')

import os
import math

class DiskSetupDialog(Gtk.Dialog):
    __gtype_name__ = "DiskSetupDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated DiskSetupDialog object.
        """
        builder = get_builder('DiskSetupDialog')
        new_object = builder.get_object('disk_setup_dialog')
        new_object.finish_initializing(builder)
        return new_object
    
    def update_interface(self, settings=None, key=None):
        if key == 'configured-system': return
        self.ui.createImageButton.set_sensitive(self.settings.get_string('disk-file-location') == '')
        self.ui.exportImageButton.set_sensitive(self.settings.get_string('disk-file-location') != '')

        self.ui.formatImageButton.set_sensitive(self.settings.get_string('disk-file-location') != '')
        self.ui.resizeImageButton.set_sensitive(False)  #self.settings.get_string('disk-file-location') != '') Not implemented yet

        if self.settings.get_string('disk-file-location') != '':
            self.ui.imagePathLabel.set_text(self.settings.get_string('disk-file-location'))
            if not diskctl.unlocked():
                try:
                    if not diskctl.decrypt_image(): return # Didn't get user authorization
                except:
                    return
            diskctl.setup_loop()
            diskctl.delete_loop()
            self.ui.fileSystemType.set_text(diskctl.fsType)
            self.ui.fileSystemSize.set_text(GLib.format_size(diskctl.fsSize))

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a DiskSetupDialog object with it in order to
        finish initializing the start of the new DiskSetupDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)

        self.settings = Gio.Settings("net.launchpad.lockbox")
        self.settings.connect('changed', self.update_interface)
        
        self.update_interface()
    
    def create_image(self, widget):
        msg = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.OTHER,
                            Gtk.ButtonsType.OK_CANCEL, "Create image")
        msg.set_title(_("Create a new image"))
        msg.set_markup(_("<big><b>Enter image data</b></big>"))
    
        grid = Gtk.Grid()
        grid.set_row_spacing(4)
        grid.set_column_spacing(4)
    
        label = Gtk.Label(_('Size:'))
        label.set_halign(Gtk.Align.START)
        grid.attach(label, 0, 0, 1, 1)
        sizeSpinButton = Gtk.SpinButton.new_with_range(100, 99999999999, 1)
        sizeSpinButton.set_digits(0)
        sizeSpinButton.set_numeric(True)
        sizeSpinButton.set_value(100)
        grid.attach(sizeSpinButton, 1, 0, 1, 1)
        label = Gtk.Label('MB')
        label.set_halign(Gtk.Align.START)
        grid.attach(label, 2, 0, 1, 1)
    
        label = Gtk.Label(_('Filesystem:'))
        label.set_halign(Gtk.Align.START)
        grid.attach(label, 0, 1, 1, 1)
        filesystemCombo = Gtk.ComboBoxText()
        filesystemCombo.append('ext2', 'Ext2')
        filesystemCombo.append('ext3', 'Ext3')
        filesystemCombo.append('ext4', 'Ext4')
        filesystemCombo.set_active(2)
        grid.attach(filesystemCombo, 1, 1, 2, 1)
    
        msg.get_message_area().pack_end(grid, False, True, 0)
        grid.show_all()
    
        result = msg.run()
        size = math.floor(sizeSpinButton.get_value())
        filesystem = filesystemCombo.get_active_id()
        msg.destroy()
        del msg
    
        if result != Gtk.ResponseType.OK: return
        
        try:
            temp_path = diskctl.get_temp_path(self, _("Cannot create image"))
            img_path = temp_path + '/image.img'
            user_data_dir = diskctl.get_data_path(self, _("Cannot create image"))
        except:
            return

        try:
            imgfile = open(img_path, 'wb')
            zerofile = open('/dev/zero', 'rb')
        except:
            MessageBox.error_msg(self, _("Cannot create image"), _("Image file creation failed"))
            return

        imgfile.write(zerofile.read(int(size)*1024*1024))
        
        imgfile.close()
        zerofile.close()
        
        diskctl.encryptOnLoopDelete = False

        try:
            diskctl.setup_loop(get_volume=False)
        except:
            MessageBox.error_msg(self, _("Cannot create image"), _("Loop device setup failed"))
            os.remove(img_path)
            return

        try:
            diskctl.format_device(filesystem, _("Lockbox disk"))
        except:
            MessageBox.error_msg(self, _("Cannot create image"), _("Unable to complete the format"))
            os.remove(img_path)
            raise
        
        diskctl.delete_loop()
        
        diskctl.encryptOnLoopDelete = True
        
        try:
            diskctl.encrypt_image(img_path, user_data_dir + '/image.img')
        except:
            MessageBox.error_msg(self, _("Cannot create image"), _("Image encryption failed"))
            os.remove(img_path)
            raise

        self.settings.set_string('disk-file-location', user_data_dir + '/image.img')
    
    def import_image(self, widget):
        if self.settings.get_string('disk-file-location') != '':
            result = MessageBox.warning_question(self, _("Confirm import"), _("Do you really want to overwrite the current image?"))
            if result != Gtk.ResponseType.YES: return
        
        dlg = Gtk.FileChooserDialog(_("Import image"), self, Gtk.FileChooserAction.OPEN, [ Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.OK ])
        result = dlg.run()
        
        filename = dlg.get_filename()
        dlg.destroy()
        
        if result != Gtk.ResponseType.OK: return
        
        try:
            if diskctl.mounted: diskctl.unmount()
            while diskctl.loopRef > 0: diskctl.delete_loop()
        except:
            MessageBox.error_msg(self, _("Cannot import image"), _("Unmount operation failed"))
            return
        
        try:
            diskctl.decrypt_image(filename)
            if self.settings.get_string('disk-file-location') != '':
                diskctl.encrypt_image()
            else:
                diskctl.encrypt_image(dstPath = diskctl.get_data_path() + '/image.img')
                self.settings.set_string('disk-file-location', diskctl.get_data_path() + '/image.img')
        except:
            MessageBox.error_msg(self, _("Cannot import image"), _("Encryption error"))
            raise
    
    def export_image(self, widget):
        dlg = Gtk.FileChooserDialog(_("Export image"), self, Gtk.FileChooserAction.SAVE, [ Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.OK ])
        result = dlg.run()
        
        filename = dlg.get_filename()
        dlg.destroy()
        
        if result != Gtk.ResponseType.OK: return
        
        if not diskctl.unlocked():
            try:
                diskctl.decrypt_image()
            except:
                MessageBox.error_msg(self, _("Cannot export image"), _("Operation failed or cancelled by the user"))
                return
        else:
            diskctl.encryptOnLoopDelete = False
            diskctl.unmount()
            diskctl.encryptOnLoopDelete = True
        
        try:
            diskctl.encrypt_image(dstPath=filename, keyFpr='')
        except:
            MessageBox.error_msg(self, _("Cannot export image"), _("Encryption error"))
            return
    
    def format_image(self, widget):
        msg = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.WARNING,
                                Gtk.ButtonsType.OK_CANCEL, "Create image")
        msg.set_title(_("Create a new image"))
        msg.set_markup(_("<big><b>Enter image data</b></big>\nWarning: this will overwrite all data in the current image"))
    
        grid = Gtk.Grid()
        grid.set_row_spacing(4)
        grid.set_column_spacing(4)
    
        label = Gtk.Label(_('Filesystem:'))
        label.set_halign(Gtk.Align.START)
        grid.attach(label, 0, 0, 1, 1)
        filesystemCombo = Gtk.ComboBoxText()
        filesystemCombo.append('ext2', 'Ext2')
        filesystemCombo.append('ext3', 'Ext3')
        filesystemCombo.append('ext4', 'Ext4')
        filesystemCombo.set_active(2)
        grid.attach(filesystemCombo, 1, 0, 2, 1)
    
        msg.get_message_area().pack_end(grid, False, True, 0)
        grid.show_all()
    
        result = msg.run()
        filesystem = filesystemCombo.get_active_id()
        msg.destroy()
        del msg
    
        if result != Gtk.ResponseType.OK: return

        if not diskctl.unlocked():
            try:
                diskctl.decrypt_image()
            except:
                MessageBox.error_msg(self, _("Cannot format image"), _("Operation failed or cancelled by the user"))
                return
        diskctl.setup_loop()
        try:
            diskctl.format_device(filesystem, _("Lockbox disk"))
        except:
            MessageBox.error_msg(self, _("Cannot format image"), _("Operation failed"))
        
        self.update_interface()
        while diskctl.loopOn:
            diskctl.delete_loop()

if __name__ == "__main__":
    dialog = DiskSetupDialog()
    dialog.show()
    Gtk.main()
