# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Fabio Massaioli fabio.massaioli@virgilio.it
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('lockbox')

import sys

from gi.repository import Gio, GLib # pylint: disable=E0611
import logging
logger = logging.getLogger('lockbox')

from lockbox_lib import Window
from lockbox.AboutLockboxDialog import AboutLockboxDialog
from lockbox.EncryptionSetupDialog import EncryptionSetupDialog
from lockbox.DiskSetupDialog import DiskSetupDialog
from lockbox.PreferencesDialog import PreferencesDialog
import diskctl, MessageBox

desktopEntryTemplate = """[Desktop Entry]
Name=Lockbox (background mode)
Comment=Run Lockbox in background mode
Categories=GNOME;Utility;
Exec=%s -b
Icon=lockbox
Terminal=false
Type=Application
X-GNOME-Autostart-enabled=%s"""

# See lockbox_lib.Window.py for more details about how this class works
class LockboxWindow(Window):
    __gtype_name__ = "LockboxWindow"
    
    def close_window(self, widget, event=None):
        return self.hide_on_delete()
    
    def quit_application(self, widget=None, data=None):
        self.get_application().quit()
        
    def setup_disk(self, widget):
        if self.settings.get_string('key-fingerprint') == '':
            MessageBox.error_msg(self, _("Encryption not configured yet"), _("You must configure the encryption system before creating the disk image"))
            return

        dialog = self.DiskSetup()
        dialog.set_transient_for(self)
        dialog.show_all()
        dialog.run()
        dialog.destroy()
        del dialog
    
    def setup_encryption(self, widget):
        dialog = self.EncryptionSetup()
        dialog.set_transient_for(self)
        dialog.show_all()
        dialog.run()
        dialog.destroy()
        del dialog
    
    def edit_preferences(self, widget=None, data=None):
        dialog = self.Preferences()
        dialog.set_transient_for(self)
        dialog.show_all()
        dialog.run()
        dialog.destroy()
        del dialog
    
    def switch_mount_state(self, widget=None, data=None):
        if diskctl.mounted: diskctl.unmount()
        else: diskctl.mount()
    
    def toggle_mount_state(self, obj, pspec):
        if self.ui.mountSwitch.get_active() != diskctl.mounted: self.switch_mount_state()
        if self.ui.mountSwitch.get_active() != diskctl.mounted: self.ui.mountSwitch.set_active(diskctl.mounted)
    
    def mount_state_changed(self, mounted):
        self.indicatorManager.refresh_menu()
        self.ui.mountSwitch.set_active(mounted)
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(LockboxWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutLockboxDialog
        self.EncryptionSetup = EncryptionSetupDialog
        self.DiskSetup = DiskSetupDialog
        self.Preferences = PreferencesDialog
        
        # Code for other initialization actions should be added here.
        self.on_preferences_changed(self.settings, 'autostart')
        
        diskctl.mountCallbacks.append(self.mount_state_changed)
        self.ui.mountSwitch.connect('notify::active', self.toggle_mount_state)
        
        self.currentKey = self.settings.get_string('key-fingerprint')
        
        try:
            sk = diskctl.gpg_context.get_key(self.settings.get_string('key-fingerprint'), True)
            del sk
        except:
            self.settings.set_string('key-fingerprint', '')
        
        if self.settings.get_string('disk-file-location') != '':
            if not Gio.file_parse_name(self.settings.get_string('disk-file-location')).query_exists(None):
                self.settings.set_string('disk-file-location', '')

        self.ui.mountSwitch.set_sensitive(self.settings.get_boolean('configured-system'))

    def on_preferences_changed(self, settings, key):
        if key == 'configured-system': return

        if self.settings.get_string('key-fingerprint') != '' and self.settings.get_string('disk-file-location') != '':
            self.settings.set_boolean('configured-system', True)
            self.ui.mountSwitch.set_sensitive(True)
            self.indicatorManager.refresh_menu()
        else:
            self.settings.set_boolean('configured-system', False)
            self.ui.mountSwitch.set_sensitive(False)
            self.indicatorManager.refresh_menu()
        
        if key == 'autostart':
            se = open(GLib.get_user_config_dir() + '/autostart/lockbox.desktop', 'w')
            se.write(desktopEntryTemplate %
                (sys.argv[0] if GLib.path_is_absolute(sys.argv[0]) else GLib.build_pathv('/', [GLib.get_current_dir(), sys.argv[0]]),
                 'true' if self.settings.get_boolean('autostart') else 'false'))
            se.close()
            return
        elif (key == 'key-fingerprint') and self.settings.get_boolean('configured-system'):
            try:
                was_unlocked = diskctl.unlocked()
                if not was_unlocked:
                    if not diskctl.decrypt_image(): return
                else:
                    diskctl.unmount()
                    while diskctl.loopOn:
                        diskctl.delete_loop()
                diskctl.encrypt_image()
            except:
                MessageBox.error_msg(self, _("Cannot update encryption"), _("Crypto backend error"))
                return

