# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Fabio Massaioli fabio.massaioli@virgilio.it
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import optparse

import gettext
from gettext import gettext as _
gettext.textdomain('lockbox')

from gi.repository import Gtk, Gdk # pylint: disable=E0611

from lockbox import LockboxWindow, diskctl

from lockbox_lib import set_up_logging, get_version

hide_window = False

def parse_options():
    global hide_window

    """Support for command line options"""
    parser = optparse.OptionParser(version="%%prog %s" % get_version())
    parser.add_option(
        "-v", "--verbose", action="count", dest="verbose",
        help=_("Show debug messages (-vv debugs lockbox_lib also)"))
    parser.add_option(
        "-b", "--background", action="store_true", dest="background",
        help=_("Start application in background mode (only show the indicator)"), default=False)
    (options, args) = parser.parse_args()

    set_up_logging(options)
    
    hide_window = options.background
    return args

def application_activated(app):
    wlist = app.get_windows()
    
    if len(wlist):
        for w in wlist:
            w.present()
    else:
        window = LockboxWindow.LockboxWindow()
        app.add_window(window)
        window.show_all()

        if hide_window: window.hide()

def main():
    'constructor for your class instances'
    args = parse_options()

    app = Gtk.Application.new("net.launchpad.lockbox", 0)
    app.connect('activate', application_activated)

    # Run the application.
    Gdk.threads_init()
    app.run(args)

    try:
        diskctl.unmount()
        while diskctl.loopOn:
            diskctl.delete_loop()
    except:
        pass

