# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Fabio Massaioli fabio.massaioli@virgilio.it
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
from gi.repository import Gtk, Gio, GLib
from gettext import gettext as _
from io import BytesIO
import MessageBox
import gpgme
import zlib

encryptOnLoopDelete = True

volumeMonitor = Gio.VolumeMonitor.get()

loopOn = False
loopRef = 0
loopObject = ''
blockProxy = None
loopProxy = None
loopDevice = ''
loopVolume = None
loopSignalConnection = None

fsType = ''
fsSize = -1

mounted = False
mountPoint = ''
mountCallbacks = []
mountObject = None
mountResult = None
mountSignalConnection = None

gpg_context = gpgme.Context()

def get_settings():
    return Gio.Settings('net.launchpad.lockbox')
    
def get_temp_path(window=None, error_msg=_("Fatal error")):
    temp_path = GLib.get_tmp_dir() + '/lockbox-' + GLib.get_user_name()
    
    tempdir = Gio.file_parse_name(temp_path)

    try:
        tempdir.make_directory(None)
        os.chmod(temp_path, 0700)
    except:
        if not tempdir.query_exists(None):
            MessageBox.error_msg(window, error_msg, _("Directory creation failed (%s)") % temp_path)
            raise

    return temp_path

def get_data_path(window=None, error_msg=_("Fatal error")):
    user_data_path = GLib.get_user_config_dir() + '/lockbox'
    
    datadir = Gio.file_parse_name(user_data_path)

    try:
        datadir.make_directory(None)
    except:
        if not datadir.query_exists(None):
            MessageBox.error_msg(window, error_msg, _("Directory creation failed (%s)") % user_data_path)
            raise
    
    return user_data_path

def unlocked():
    return Gio.file_parse_name(get_temp_path() + '/image.img').query_exists(None)

def loop_deleted(volume):
    global loopOn, loopRef, loopObject, blockProxy, loopProxy, loopSignalConnection, loopDevice, loopVolume
    
    print "Volume removed"
    
    loopVolume.disconnect(loopSignalConnection)
    
    loopOn = False
    loopRef = 0
    loopObject = ''
    blockProxy = None
    loopProxy = None
    loopDevice = ''
    loopVolume = None
    loopSignalConnection = None
        
    if encryptOnLoopDelete: encrypt_image()

def get_loop_volume():
    Gtk.main_iteration()
    for v in volumeMonitor.get_volumes():
        if v.get_name() == blockProxy.get_cached_property('IdLabel').get_string():
            return v
    return None

def setup_loop(filePath=get_temp_path() + '/image.img', get_volume=True):
    global loopOn, loopRef, loopObject, blockProxy, loopProxy, loopSignalConnection, loopDevice, loopVolume, fsType, fsSize

    if loopOn:
        loopRef = loopRef+1
        return

    bus = Gio.bus_get_sync(Gio.BusType.SYSTEM, None)
    
    fd = os.open(filePath, os.O_RDWR)
    
    try:
        loopObject = bus.call_with_unix_fd_list_sync(
                        'org.freedesktop.UDisks2',
                        '/org/freedesktop/UDisks2/Manager',
                        'org.freedesktop.UDisks2.Manager',
                        'LoopSetup',
                        GLib.Variant('(ha{sv})', ( 0, { 'no-part-scan': GLib.Variant.new_boolean(True), 'read-only': GLib.Variant.new_boolean(False) } )),
                        None,
                        0,
                        -1,
                        Gio.UnixFDList.new_from_array([ fd ]),
                        None)[0][0]
    except:
        os.close(fd)
        raise
    
    loopOn = True
    loopRef += 1
    
    blockProxy = Gio.DBusProxy.new_sync(bus, 0, None, 'org.freedesktop.UDisks2', loopObject, 'org.freedesktop.UDisks2.Block', None)
    loopDevice = blockProxy.get_cached_property('Device').get_bytestring()
    fsSize = blockProxy.get_cached_property('Size').get_uint64()
    fsType = blockProxy.get_cached_property('IdType').get_string()
    
    loopProxy = Gio.DBusProxy.new_sync(bus, 0, None, 'org.freedesktop.UDisks2', loopObject, 'org.freedesktop.UDisks2.Loop', None)
    try:
        loopProxy.call_sync('SetAutoclear', GLib.Variant.new('(ba{sv})', (True, {})), 0, -1, None)
    except:
        pass

    while loopVolume == None and get_volume:
      loopVolume = get_loop_volume()
    
    if loopVolume:
        loopSignalConnection = loopVolume.connect('removed', loop_deleted)

def delete_loop():
    global loopOn, loopRef, loopObject, blockProxy, loopProxy, loopSignalConnection, loopDevice, loopVolume, fsType, fsSize

    if not loopOn: return
    loopRef -= 1

    if loopRef > 0: return
    if mounted:
        loopRef = 1
        return

    if loopVolume: loopVolume.disconnect(loopSignalConnection)    
    try:
        loopProxy.call_sync('Delete', GLib.Variant('(a{sv})', ({},)), 0, -1, None)
    except:
        if loopVolume:
            loopSignalConnection = loopVolume.connect('removed', loop_deleted)
        raise

    loopOn = False
    loopRef = 0
    loopObject = ''
    loopDevice = ''
    blockProxy = None
    loopVolume = None
    loopSignalConnection = None
    loopProxy = None
    
    if encryptOnLoopDelete: encrypt_image()
    return

def format_device(filesystem, label):
    global blockProxy, fsType, fsSize
    
    if not unlocked(): raise
    
    setup_loop()
    if mounted: unmount()

    blockProxy.call_sync('Format', GLib.Variant('(sa{sv})',
                         (filesystem, {'label': GLib.Variant.new_string(label), 'take-ownership': GLib.Variant.new_boolean(True)})), 0, -1, None)
    fsType = filesystem
    
    delete_loop()

def volume_unmounted(mount):
    global loopVolume, loopProxy, mounted, mountPoint, mountObject, mountSignalConnection

    try:
        loopProxy.call_sync('SetAutoclear', GLib.Variant.new('(ba{sv})', (False, {})), 0, -1, None)
    except:
        pass
    
    print("Unmounted")

    mountObject.disconnect(mountSignalConnection)
    mountSignalConnection = None
    mountObject = None
    mounted = False
    mountPoint = ''
    
    try:
        loopProxy.call_sync('SetAutoclear', GLib.Variant.new('(ba{sv})', (True, {})), 0, -1, None)
    except:
        pass

    while loopOn:
        try:
            delete_loop()
            break
        except:
            if loopRef > 1: break
            continue
    
    for cb in mountCallbacks:
        try: cb(False)
        except: pass

def complete_mount(obj, result, data=None):
    global mountResult
    mountResult = result

def mount():
    global loopVolume, mounted, mountPoint, mountResult, mountObject, mountSignalConnection
    
    if mounted: return
    if not unlocked():
        try:
            if not decrypt_image(): return
        except:
            return

    setup_loop()
    
    mount_op = Gtk.MountOperation()
    mountResult = None
    loopVolume.mount(0, mount_op, None, complete_mount, None)
    
    while mountResult == None: Gtk.main_iteration()
    try:
        if not loopVolume.mount_finish(mountResult): raise
    except:
        MessageBox.error_msg(None, _("Cannot mount image"), _("Mount error"))
        delete_loop()
        raise

    mountObject = loopVolume.get_mount()
    mountPoint = mountObject.get_root().get_path()
    mounted = True
    
    mountSignalConnection = mountObject.connect('unmounted', volume_unmounted)

    for cb in mountCallbacks:
        try: cb(True)
        except: pass

def unmount():
    global loopVolume, loopProxy, mounted, mountPoint, mountObject, mountResult, mountSignalConnection
    
    if not mounted: return
    
    try:
        loopProxy.call_sync('SetAutoclear', GLib.Variant.new('(ba{sv})', (False, {})), 0, -1, None)
    except:
        pass
    
    mountObject.disconnect(mountSignalConnection)
    mountSignalConnection = None
    
    mount_op = Gtk.MountOperation()
    mountResult = None
    mountObject.unmount_with_operation(0, mount_op, None, complete_mount, None)
    
    while mountResult == None: Gtk.main_iteration()
    
    try:
        if not mountObject.unmount_with_operation_finish(mountResult): raise
    except:
        MessageBox.error_msg(None, _("Cannot unmount image"), _("Check whether the virtual disk is currently in use"))
        raise

    mountObject = None
    mountPoint = ''
    mounted = False
    delete_loop()
    try:
        loopProxy.call_sync('SetAutoclear', GLib.Variant.new('(ba{sv})', (True, {})), 0, -1, None)
    except:
        pass

    for cb in mountCallbacks:
        try: cb(False)
        except: pass

def encrypt_image(srcPath=get_temp_path() + '/image.img', dstPath=None, keyFpr=None, remove=True, show_activity_dialog=False):
    settings = get_settings()
    if dstPath == None: dstPath = settings.get_string('disk-file-location')
    if keyFpr == None: keyFpr = settings.get_string('key-fingerprint')
    
    srcFile = open(srcPath, 'rb')
    
    try:
        dstFile = open(dstPath, 'wb')
    except:
        srcFile.close()
        raise
    
    try:
        imgData = zlib.compress(srcFile.read(), 9)
    except:
        srcFile.close()
        dstFile.close()
        raise
    
    srcFile.close()
    
    key = None
    if keyFpr != '':
        try:
            key = gpg_context.get_key(keyFpr, False)
        except:
            dstFile.close()
            raise
    
    plain = BytesIO(imgData)
    cipher = BytesIO()
    
    try:
        gpg_context.encrypt([ key ] if key != None else None, 0, plain, cipher)
    except:
        dstFile.close()
        raise
    
    cipher.seek(0)
    dstFile.write(cipher.read())
    dstFile.close()
    
    if remove: os.remove(srcPath)

def decrypt_image(srcPath=None, dstPath=get_temp_path() + '/image.img', show_activity_dialog=False):
    settings = get_settings()
    if srcPath == None: srcPath = settings.get_string('disk-file-location')
    
    result = MessageBox.warning_question(None, _("Lockbox authorization request"), _("Do you really want to unlock the disk?"))
    if result != Gtk.ResponseType.YES: return False

    srcFile = open(srcPath, 'rb')

    try:
        dstFile = open(dstPath, 'wb')
    except:
        srcFile.close()
        raise
    
    cipher = BytesIO(srcFile.read())
    plain = BytesIO()
    
    srcFile.close()
    
    try:
        gpg_context.decrypt(cipher, plain)
    except:
        dstFile.close()
        os.remove(dstPath)
        raise
    
    plain.seek(0)
    
    try:
        imgData = zlib.decompress(plain.read())
    except:
        dstFile.close()
        os.remove(dstPath)
        raise
    
    dstFile.write(imgData)
    dstFile.close()
    
    return True

