# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Fabio Massaioli fabio.massaioli@virgilio.it
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

"""Code to add AppIndicator."""

from gi.repository import Gtk # pylint: disable=E0611
from gi.repository import AppIndicator3 # pylint: disable=E0611

from lockbox_lib.helpers import get_media_file
import diskctl

import gettext
from gettext import gettext as _
gettext.textdomain('lockbox')

class Indicator:
    def show_window(self, widget):
        self.window.present()
    
    def refresh_menu(self, widget=None, user_data=None):
        self.mount.set_sensitive(self.window.settings.get_boolean('configured-system'))
        self.mount.set_label(_("Lock disk") if diskctl.mounted else _("Unlock disk"))
    
    def quit_application(self, widget):
        self.window.get_application().quit()

    def __init__(self, window):
        self.window = window
        self.indicator = AppIndicator3.Indicator.new('lockbox', '', AppIndicator3.IndicatorCategory.APPLICATION_STATUS)
        self.indicator.set_status(AppIndicator3.IndicatorStatus.ACTIVE)

        icon_uri = get_media_file("lockbox-indicator.svg")
        icon_path = icon_uri.replace("file:///", '')
        self.indicator.set_icon(icon_path)
        
        self.indicator.set_title(_("Lockbox"))

        #Uncomment and choose an icon for attention state. 
        #self.indicator.set_attention_icon("ICON-NAME")
        
        self.menu = Gtk.Menu()

        # Add items to Menu and connect signals.
        
        self.mount = Gtk.MenuItem(_("Unlock disk"))
        self.mount.set_sensitive(window.settings.get_boolean('configured-system'))
        self.mount.connect('activate', self.window.switch_mount_state)
        self.mount.show()
        self.menu.append(self.mount)
        
        sep = Gtk.SeparatorMenuItem()
        sep.show()
        self.menu.append(sep)
        
        #Adding preferences button 
        #window represents the main Window object of your app
        self.preferences = Gtk.MenuItem(_("Preferences"))
        self.preferences.connect("activate", self.show_window)
        self.preferences.show()
        self.menu.append(self.preferences)

        self.quit = Gtk.MenuItem(_("Quit application"))
        self.quit.connect("activate", self.quit_application)
        self.quit.show()
        self.menu.append(self.quit)

        # Add more items here                           

        self.menu.show()
        self.indicator.set_menu(self.menu)
    
def new_application_indicator(window):
    ind = Indicator(window)
    window.indicatorManager = ind
    return ind.indicator
